function pink = pinkNoise(iterations, economy, plotFlag)
% sig = pinkNoise(iterations);
% Pink noise generator
%
% G. Clifford 2004 gari AT mit DOT edu  adapted from 
%         paul.kellett AT maxim.abel.co.uk
% 

if nargin<1
iterations=1000;
end
if nargin<2
economy=0;
end
if nargin<3
plotFlag=0;
end


% intialise
white = rand(1,iterations); white=white-mean(white);
pink  = zeros(1,iterations);
b0=0;b1=0;b2=0;b3=0;b4=0;b5=0;b6=0;

if economy==0; % slow & accurate version

for(i=1:length(white)) % loop over each element to create a new one
b0 = 0.99886 * b0 + white(i) * 0.0555179;
b1 = 0.99332 * b1 + white(i) * 0.0750759;
b2 = 0.96900 * b2 + white(i) * 0.1538520;
b3 = 0.86650 * b3 + white(i) * 0.3104856;
b4 = 0.55000 * b4 + white(i) * 0.5329522;
b5 = -0.7616 * b5 - white(i) * 0.0168980;
pink(i) = b0 + b1 + b2 + b3 + b4 + b5 + b6 + white(i) * 0.5362;
b6 = white(i) * 0.115926;
end

end

% An 'economy' version with accuracy of +/-0.5dB 

if economy==1

for(i=1:length(white)) % loop over each element to create a new one
b0 = 0.99765 * b0 + white(i) * 0.0990460;
b1 = 0.96300 * b1 + white(i) * 0.2965164;
b2 = 0.57000 * b2 + white(i) * 1.0526913;
pink(i) = b0 + b1 + b2 + white(i) * 0.1848; 
end

end

if economy>1
% These have approximately equiripple error in decibels from 20hz to 20khz at a 44.1khz sampling rate.

%1st order, ~ +/- 3 dB error (not recommended!)
num = [0.05338071119116 -0.03752455712906]
den = [1.00000000000000 -0.97712493947102]

%2nd order, ~ +/- 0.9 dB error
num = [ 0.04957526213389 -0.06305581334498 0.01483220320740 ];
den = [ 1.00000000000000 -1.80116083982126 0.80257737639225 ];
pink = filtfilt(den,num,white);
end

% just plotting the data
if plotFlag>0

figure;
subplot(2,1,1)
plot(pink,'+--');
axis([0 length(pink) min(pink)-(abs(min(pink))/20) max(pink)+(abs(max(pink))/20) ])
title('Pink Noise');
grid on;
subplot(2,1,2)
psd(pink)

end

